RAW_TREE_NAME = "raw"
TIME_NS_TREE_NAME = "time_ns"
BSD_TREE_NAME = "bsd"
FSN_TREE_NAME = "fixed_spill_number"
PEU_TREE_NAME = "peu"
FLAGS_TREE_NAME = "detector_flags"
THRESHOLD_TREE_NAME = "hit_threshold"
TEMPHUM_TREE_NAME = "temp_hum"
HISTORY_TREE_NAME = "history"
